package cn.ibizlab.core.uaa.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.uaa.domain.SysAuthLog;
import cn.ibizlab.core.uaa.filter.SysAuthLogSearchContext;
import cn.ibizlab.core.uaa.service.ISysAuthLogService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.uaa.mapper.SysAuthLogMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[认证日志] 服务对象接口实现
 */
@Slf4j
@Service("SysAuthLogServiceImpl")
public class SysAuthLogServiceImpl extends ServiceImpl<SysAuthLogMapper, SysAuthLog> implements ISysAuthLogService {

    @Autowired
    @Lazy
    ISysAuthLogService proxyService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(SysAuthLog et) {
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getLogid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<SysAuthLog> list) {
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(SysAuthLog et) {
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("logid", et.getLogid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getLogid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<SysAuthLog> list) {
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public SysAuthLog get(String key) {
        SysAuthLog et = getById(key);
        if(et == null){
            et = new SysAuthLog();
            et.setLogid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public SysAuthLog getDraft(SysAuthLog et) {
        return et;
    }

    @Override
    public boolean checkKey(SysAuthLog et) {
        return (!ObjectUtils.isEmpty(et.getLogid())) && (!Objects.isNull(this.getById(et.getLogid())));
    }
    @Override
    @Transactional
    public boolean save(SysAuthLog et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(SysAuthLog et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? proxyService.update(et) : proxyService.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<SysAuthLog> list) {
        List<SysAuthLog> create = new ArrayList<>();
        List<SysAuthLog> update = new ArrayList<>();
        for (SysAuthLog et : list) {
            if (ObjectUtils.isEmpty(et.getLogid()) || ObjectUtils.isEmpty(getById(et.getLogid()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            proxyService.createBatch(create);
        }
        if (update.size() > 0) {
            proxyService.updateBatch(update);
        }
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<SysAuthLog> list) {
        List<SysAuthLog> create = new ArrayList<>();
        List<SysAuthLog> update = new ArrayList<>();
        for (SysAuthLog et : list) {
            if (ObjectUtils.isEmpty(et.getLogid()) || ObjectUtils.isEmpty(getById(et.getLogid()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            proxyService.createBatch(create);
        }
        if (update.size() > 0) {
            proxyService.updateBatch(update);
        }
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<SysAuthLog> searchDefault(SysAuthLogSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SysAuthLog> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<SysAuthLog>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }





}



